<?php
/*+**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 ************************************************************************************/
require_once('data/CRMEntity.php');
require_once('data/Tracker.php');

class Planetauthorize extends CRMEntity {
	var $db, $log; // Used in class functions of CRMEntity
	var $table_name = 'vtiger_planetauthorize';
	var $table_index= 'planetauthorizeid';
	var $column_fields = Array();
	/** Indicator if this is a custom module or standard module */
	var $IsCustomModule = true;

	/**
	 * Mandatory table for supporting custom fields.
	 */
	var $customFieldTable = Array('vtiger_planetauthorizecf', 'planetauthorizeid');

	/**
	 * Mandatory for Saving, Include tables related to this module.
	 */
	var $tab_name = Array('vtiger_crmentity', 'vtiger_planetauthorize', 'vtiger_planetauthorizecf');

	/**
	 * Mandatory for Saving, Include tablename and tablekey columnname here.
	 */
	var $tab_name_index = Array(
		'vtiger_crmentity' => 'crmid',
		'vtiger_planetauthorize'   => 'planetauthorizeid',
	    'vtiger_planetauthorizecf' => 'planetauthorizeid');

	/**
	 * Mandatory for Listing (Related listview)
	 */
	var $list_fields = Array (
		/* Format: Field Label => Array(tablename, columnname) */
		// tablename should not have prefix 'vtiger_'
		'TransactionID'=> Array('planetauthorize', 'transaction_id'),
        'PaymentType'=>Array('planetauthorize'=>'payment_type'),
        'TransactionType'=>Array('planetauthorize'=>'transaction_type'),
        'TransactionAmount'=>Array('planetauthorize'=>'transaction_amount'),
        'TransactionDate'=>Array('planetauthorize'=> 'transaction_date'),
        'response'=>Array('planetauthorize'=>'response'),
		'Assigned To' => Array('crmentity','smownerid'),
		'Firstname' => Array('planetauthorize','bill_firstname'),
		'Lastname' => Array('planetauthorize','bill_lastname'),
		'Description' => Array('planetauthorize','description'),
        'Avsresponse'=> Array('planetauthorize','avsresponse'),
        'Cvvresponse'=> Array('planetauthorize','cvvresponse'),
        'orderid' => Array('planetauthorize','orderid'),
        'ResponseCode' => Array('planetauthorize','response_code'),
        'CustomerVaultId' => Array('planetauthorize','customer_vault_id'),
        'RecurringPlans' =>  Array('planetauthorize','recurring_plans')
	);
	var $list_fields_name = Array(
		/* Format: Field Label => fieldname */
		'TransactionID'=> 'transaction_id',
        'PaymentType'=>'payment_type',
        'TransactionType'=>'transaction_type',
        'TransactionAmount'=>'transaction_amount',
        'TransactionDate'=>'transaction_date',
        'response'=>'response',
		'Assigned To' => 'assigned_user_id',
		'Firstname' => 'bill_firstname',
		'Lastname' => 'bill_lastname',
        'Description' => 'description',
        'Avsresponse'=> 'avsresponse',
        'Cvvresponse'=> 'cvvresponse',
        'orderid' => 'orderid',
        'ResponseCode' => 'response_code',
        'CustomerVaultId' => 'customer_vault_id',
        'RecurringPlans' => 'recurring_plan'
	);

	// Make the field link to detail view from list view (Fieldname)
	var $list_link_field = 'transaction_id';

	// For Popup listview and UI type support
	var $search_fields = Array(
		/* Format: Field Label => Array(tablename, columnname) */
		// tablename should not have prefix 'vtiger_'
		'TransactionID'=> Array('planetauthorize', 'transaction_id'),
		'TransactionType'=> Array('planetauthorize', 'transaction_type'),
		'PaymentType'=> Array('planetauthorize', 'payment_type'),
		'TransactionDate'=> Array('planetauthorize', 'transaction_date'),
		'TransactionAmount'=> Array('planetauthorize', 'transaction_amount'),
		'response'=> Array('planetauthorize', 'response'),
        'Firstname' => Array('planetauthorize','bill_firstname'),
        'Lastname' => Array('planetauthorize','bill_lastname')
	);
	var $search_fields_name = Array(
		/* Format: Field Label => fieldname */
		'TransactionID'=> 'transaction_id',
		'TransactionType'=> 'transaction_type',
		'PaymentType'=> 'payment_type',
		'TransactionDate'=> 'transaction_date',
		'TransactionAmount'=> 'transaction_amount',
        'response'=>'response',
        'Firstname' => 'bill_firstname',
        'Lastname' => 'bill_lastname'
	);

	// For Popup window record selection
	var $popup_fields = Array('transaction_id');

	// Placeholder for sort fields - All the fields will be initialized for Sorting through initSortFields
	var $sortby_fields = Array();

	// For Alphabetical search
	var $def_basicsearch_col = 'transaction_type';

	// Column value to use on detail view record text display
	var $def_detailview_recname = 'transaction_type';

	// Required Information for enabling Import feature
	var $required_fields = Array('transaction_id'=>1);

	// Callback function list during Importing
	var $special_functions = Array('set_import_assigned_user');

	var $default_order_by = 'transaction_date';
	var $default_sort_order='ASC';
	// Used when enabling/disabling the mandatory fields for the module.
	// Refers to vtiger_field.fieldname values.
	var $mandatory_fields = Array('createdtime', 'modifiedtime', 'transaction_id');
    //Payment gateway url
    var $cgiurl = "https://secure.planetauthorizegateway.com/api/transact.php";
	
	function __construct() {
		global $log, $currentModule;
		$this->column_fields = getColumnFields($currentModule);
		$this->db = PearDatabase::getInstance();
		$this->log = $log;
	}

	function getSortOrder() {
		global $currentModule;

		$sortorder = $this->default_sort_order;
		if($_REQUEST['sorder']) $sortorder = $this->db->sql_escape_string($_REQUEST['sorder']);
		else if($_SESSION[$currentModule.'_Sort_Order']) 
			$sortorder = $_SESSION[$currentModule.'_Sort_Order'];

		return $sortorder;
	}

	function getOrderBy() {
		global $currentModule;
		
		$use_default_order_by = '';		
		if(PerformancePrefs::getBoolean('LISTVIEW_DEFAULT_SORTING', true)) {
			$use_default_order_by = $this->default_order_by;
		}
		
		$orderby = $use_default_order_by;
		if($_REQUEST['order_by']) $orderby = $this->db->sql_escape_string($_REQUEST['order_by']);
		else if($_SESSION[$currentModule.'_Order_By'])
			$orderby = $_SESSION[$currentModule.'_Order_By'];
		return $orderby;
	}

	function save_module($module) {
	}

	/**
	 * Return query to use based on given modulename, fieldname
	 * Useful to handle specific case handling for Popup
	 */
	function getQueryByModuleField($module, $fieldname, $srcrecord) {
		// $srcrecord could be empty
	}

	/**
	 * Get list view query (send more WHERE clause condition if required)
	 */
	function getListQuery($module, $where='') {
		$query = "SELECT vtiger_crmentity.*, $this->table_name.transaction_id,";
        $query .= "$this->table_name.transaction_type,$this->table_name.payment_type,";
        $query .= "$this->table_name.bill_firstname,$this->table_name.bill_lastname,"; 
        $query .= "$this->table_name.transaction_amount,vtiger_pa_response.value as response,";
        $query .= "$this->table_name.transaction_date,vtiger_pa_response.value as response ";

		// Select Custom Field Table Columns if present
		if(!empty($this->customFieldTable)) $query .= ", " . $this->customFieldTable[0] . ".* ";

		$query .= " FROM $this->table_name";

		$query .= "	INNER JOIN vtiger_crmentity ON vtiger_crmentity.crmid = $this->table_name.$this->table_index";

		// Consider custom table join as well.
		if(!empty($this->customFieldTable)) {
			$query .= " INNER JOIN ".$this->customFieldTable[0]." ON ".$this->customFieldTable[0].'.'.$this->customFieldTable[1] .
				      " = $this->table_name.$this->table_index"; 
		}
		$query .= " LEFT JOIN vtiger_users ON vtiger_users.id = vtiger_crmentity.smownerid";
		$query .= " LEFT JOIN vtiger_groups ON vtiger_groups.groupid = vtiger_crmentity.smownerid";
		$query .= " LEFT JOIN vtiger_pa_response ON vtiger_pa_response.id = vtiger_planetauthorize.response";

		$linkedModulesQuery = $this->db->pquery("SELECT distinct fieldname, columnname, relmodule FROM vtiger_field" .
				" INNER JOIN vtiger_fieldmodulerel ON vtiger_fieldmodulerel.fieldid = vtiger_field.fieldid" .
				" WHERE uitype='10' AND vtiger_fieldmodulerel.module=?", array($module));
		$linkedFieldsCount = $this->db->num_rows($linkedModulesQuery);
		
		for($i=0; $i<$linkedFieldsCount; $i++) {
			$related_module = $this->db->query_result($linkedModulesQuery, $i, 'relmodule');
			$fieldname = $this->db->query_result($linkedModulesQuery, $i, 'fieldname');
			$columnname = $this->db->query_result($linkedModulesQuery, $i, 'columnname');
			
			$other = CRMEntity::getInstance($related_module);
			vtlib_setup_modulevars($related_module, $other);
			
			$query .= " LEFT JOIN $other->table_name ON $other->table_name.$other->table_index = $this->table_name.$columnname";
		}

		$query .= "	WHERE vtiger_crmentity.deleted = 0 ".$where;
		$query .= $this->getListViewSecurityParameter($module);
		return $query;
	}

	/**
	 * Apply security restriction (sharing privilege) query part for List view.
	 */
	function getListViewSecurityParameter($module) {
		global $current_user;
		require('user_privileges/user_privileges_'.$current_user->id.'.php');
		require('user_privileges/sharing_privileges_'.$current_user->id.'.php');

		$sec_query = '';
		$tabid = getTabid($module);

		if($is_admin==false && $profileGlobalPermission[1] == 1 && $profileGlobalPermission[2] == 1 
			&& $defaultOrgSharingPermission[$tabid] == 3) {

				$sec_query .= " AND (vtiger_crmentity.smownerid in($current_user->id) OR vtiger_crmentity.smownerid IN 
					(
						SELECT vtiger_user2role.userid FROM vtiger_user2role 
						INNER JOIN vtiger_users ON vtiger_users.id=vtiger_user2role.userid 
						INNER JOIN vtiger_role ON vtiger_role.roleid=vtiger_user2role.roleid 
						WHERE vtiger_role.parentrole LIKE '".$current_user_parent_role_seq."::%'
					) 
					OR vtiger_crmentity.smownerid IN 
					(
						SELECT shareduserid FROM vtiger_tmp_read_user_sharing_per 
						WHERE userid=".$current_user->id." AND tabid=".$tabid."
					) 
					OR 
						(";
		
					// Build the query based on the group association of current user.
					if(sizeof($current_user_groups) > 0) {
						$sec_query .= " vtiger_groups.groupid IN (". implode(",", $current_user_groups) .") OR ";
					}
					$sec_query .= " vtiger_groups.groupid IN 
						(
							SELECT vtiger_tmp_read_group_sharing_per.sharedgroupid 
							FROM vtiger_tmp_read_group_sharing_per
							WHERE userid=".$current_user->id." and tabid=".$tabid."
						)";
				$sec_query .= ")
				)";
		}
		return $sec_query;
	}

	/**
	 * Create query to export the records.
	 */
	function create_export_query($where)
	{
		global $current_user;
		$thismodule = $_REQUEST['module'];
		
		include("include/utils/ExportUtils.php");

		//To get the Permitted fields query and the permitted fields list
		$sql = getPermittedFieldsQuery($thismodule, "detail_view");
		
		$fields_list = getFieldsListFromQuery($sql);

		$query = "SELECT $fields_list, vtiger_users.user_name AS user_name 
					FROM vtiger_crmentity INNER JOIN $this->table_name ON vtiger_crmentity.crmid=$this->table_name.$this->table_index";

		if(!empty($this->customFieldTable)) {
			$query .= " INNER JOIN ".$this->customFieldTable[0]." ON ".$this->customFieldTable[0].'.'.$this->customFieldTable[1] .
				      " = $this->table_name.$this->table_index"; 
		}

		$query .= " LEFT JOIN vtiger_groups ON vtiger_groups.groupid = vtiger_crmentity.smownerid";
		$query .= " LEFT JOIN vtiger_users ON vtiger_crmentity.smownerid = vtiger_users.id and vtiger_users.status='Active'";
		
		$linkedModulesQuery = $this->db->pquery("SELECT distinct fieldname, columnname, relmodule FROM vtiger_field" .
				" INNER JOIN vtiger_fieldmodulerel ON vtiger_fieldmodulerel.fieldid = vtiger_field.fieldid" .
				" WHERE uitype='10' AND vtiger_fieldmodulerel.module=?", array($thismodule));
		$linkedFieldsCount = $this->db->num_rows($linkedModulesQuery);

		for($i=0; $i<$linkedFieldsCount; $i++) {
			$related_module = $this->db->query_result($linkedModulesQuery, $i, 'relmodule');
			$fieldname = $this->db->query_result($linkedModulesQuery, $i, 'fieldname');
			$columnname = $this->db->query_result($linkedModulesQuery, $i, 'columnname');
			
			$other = CRMEntity::getInstance($related_module);
			vtlib_setup_modulevars($related_module, $other);
			
			$query .= " LEFT JOIN $other->table_name ON $other->table_name.$other->table_index = $this->table_name.$columnname";
		}

		$where_auto = " vtiger_crmentity.deleted=0";

		if($where != '') $query .= " WHERE ($where) AND $where_auto";
		else $query .= " WHERE $where_auto";

		require('user_privileges/user_privileges_'.$current_user->id.'.php');
		require('user_privileges/sharing_privileges_'.$current_user->id.'.php');

		// Security Check for Field Access
		if($is_admin==false && $profileGlobalPermission[1] == 1 && $profileGlobalPermission[2] == 1 && $defaultOrgSharingPermission[7] == 3)
		{
			//Added security check to get the permitted records only
			$query = $query." ".getListViewSecurityParameter($thismodule);
		}
		return $query;
	}

	/**
	 * Initialize this instance for importing.
	 */
	function initImport($module) {
		$this->db = PearDatabase::getInstance();
		$this->initImportableFields($module);
	}

	/**
	 * Create list query to be shown at the last step of the import.
	 * Called From: modules/Import/UserLastImport.php
	 */
	function create_import_query($module) {
		global $current_user;
		$query = "SELECT vtiger_crmentity.crmid, case when (vtiger_users.user_name not like '') then vtiger_users.user_name else vtiger_groups.groupname end as user_name, $this->table_name.* FROM $this->table_name
			INNER JOIN vtiger_crmentity ON vtiger_crmentity.crmid = $this->table_name.$this->table_index
			LEFT JOIN vtiger_users_last_import ON vtiger_users_last_import.bean_id=vtiger_crmentity.crmid
			LEFT JOIN vtiger_users ON vtiger_users.id = vtiger_crmentity.smownerid
			LEFT JOIN vtiger_groups ON vtiger_groups.groupid = vtiger_crmentity.smownerid
			WHERE vtiger_users_last_import.assigned_user_id='$current_user->id'
			AND vtiger_users_last_import.bean_type='$module'
			AND vtiger_users_last_import.deleted=0";
		return $query;
	}

	/**
	 * Delete the last imported records.
	 */
	function undo_import($module, $user_id) {
		global $adb;
		$count = 0;
		$query1 = "select bean_id from vtiger_users_last_import where assigned_user_id=? AND bean_type='$module' AND deleted=0";
		$result1 = $adb->pquery($query1, array($user_id)) or die("Error getting last import for undo: ".mysql_error()); 
		while ( $row1 = $adb->fetchByAssoc($result1))
		{
			$query2 = "update vtiger_crmentity set deleted=1 where crmid=?";
			$result2 = $adb->pquery($query2, array($row1['bean_id'])) or die("Error undoing last import: ".mysql_error()); 
			$count++;			
		}
		return $count;
	}
	
	/**
	 * Transform the value while exporting
	 */
	function transform_export_value($key, $value) {
		return parent::transform_export_value($key, $value);
	}

	/**
	 * Function which will set the assigned user id for import record.
	 */
	function set_import_assigned_user()
	{
		global $current_user, $adb;
		$record_user = $this->column_fields["assigned_user_id"];
		
		if($record_user != $current_user->id){
			$sqlresult = $adb->pquery("select id from vtiger_users where id = ? union select groupid as id from vtiger_groups where groupid = ?", array($record_user, $record_user));
			if($this->db->num_rows($sqlresult)!= 1) {
				$this->column_fields["assigned_user_id"] = $current_user->id;
			} else {			
				$row = $adb->fetchByAssoc($sqlresult, -1, false);
				if (isset($row['id']) && $row['id'] != -1) {
					$this->column_fields["assigned_user_id"] = $row['id'];
				} else {
					$this->column_fields["assigned_user_id"] = $current_user->id;
				}
			}
		}
	}
	
	/** 
	 * Function which will give the basic query to find duplicates
	 */
	function getDuplicatesQuery($module,$table_cols,$field_values,$ui_type_arr,$select_cols='') {
		$select_clause = "SELECT ". $this->table_name .".".$this->table_index ." AS recordid, vtiger_users_last_import.deleted,".$table_cols;

		// Select Custom Field Table Columns if present
		if(isset($this->customFieldTable)) $query .= ", " . $this->customFieldTable[0] . ".* ";

		$from_clause = " FROM $this->table_name";

		$from_clause .= "	INNER JOIN vtiger_crmentity ON vtiger_crmentity.crmid = $this->table_name.$this->table_index";

		// Consider custom table join as well.
		if(isset($this->customFieldTable)) {
			$from_clause .= " INNER JOIN ".$this->customFieldTable[0]." ON ".$this->customFieldTable[0].'.'.$this->customFieldTable[1] .
				      " = $this->table_name.$this->table_index"; 
		}
		$from_clause .= " LEFT JOIN vtiger_users ON vtiger_users.id = vtiger_crmentity.smownerid
						LEFT JOIN vtiger_groups ON vtiger_groups.groupid = vtiger_crmentity.smownerid";
		
		$where_clause = "	WHERE vtiger_crmentity.deleted = 0";
		$where_clause .= $this->getListViewSecurityParameter($module);
					
		if (isset($select_cols) && trim($select_cols) != '') {
			$sub_query = "SELECT $select_cols FROM  $this->table_name AS t " .
				" INNER JOIN vtiger_crmentity AS crm ON crm.crmid = t.".$this->table_index;
			// Consider custom table join as well.
			if(isset($this->customFieldTable)) {
				$sub_query .= " LEFT JOIN ".$this->customFieldTable[0]." tcf ON tcf.".$this->customFieldTable[1]." = t.$this->table_index";
			}
			$sub_query .= " WHERE crm.deleted=0 GROUP BY $select_cols HAVING COUNT(*)>1";	
		} else {
			$sub_query = "SELECT $table_cols $from_clause $where_clause GROUP BY $table_cols HAVING COUNT(*)>1";
		}	
		
		
		$query = $select_clause . $from_clause .
					" LEFT JOIN vtiger_users_last_import ON vtiger_users_last_import.bean_id=" . $this->table_name .".".$this->table_index .
					" INNER JOIN (" . $sub_query . ") AS temp ON ".get_on_clause($field_values,$ui_type_arr,$module) .
					$where_clause .
					" ORDER BY $table_cols,". $this->table_name .".".$this->table_index ." ASC";
					
		return $query;		
	}

	/**
	 * Invoked when special actions are performed on the module.
	 * @param String Module name
	 * @param String Event Type (module.postinstall, module.disabled, module.enabled, module.preuninstall)
	 */
	function vtlib_handler($modulename, $event_type) {
		if($event_type == 'module.postinstall') {
			// TODO Handle post installation actions
            $sql = "INSERT INTO vtiger_pa_response(id,value) VALUES(1, 'Transaction approved'),(2, 'Transaction declined'),(3, 'Error in transaction data or system error')";
            $this->db->query($sql);
            $sql = "INSERT INTO vtiger_pa_avsresponsecode(Code, Value) VALUES('0', 'AVS Not Available'),('A', 'Address Match Only'),('B', '\"'),('C', '\"'),('D', '\"\"\"'),('E', 'Not a mail/phone order'),('G', 'Non-U.S. Issuer does not participate'),('I', '\"'),('L', '\"'),('M', '\"\"\"'),('N', 'No address or ZIP match'),('O', '\"'),('P', '\"'),('R', 'Issuer system unavailable'),('S', 'Service not supported'),('U', 'Address unavailable'),('W', '9-character numeric ZIP match only'),('X', 'Exact match, 9-character numeric ZIP'),('Y', 'Exact match, 5-character numeric ZIP'),('Z', '5-character Zip match only')";
            $this->db->query($sql);
            $sql = "INSERT INTO vtiger_pa_cvvresponsecode (code, value) VALUES('M', 'CVV2/CVC2 Match'),('N', 'CVV2/CVC2 No Match'),('P', 'Not Processed'),('S', 'Merchant has indicated that CVV2/CVC2 is not present on card '),('U', 'Issuer is not certified and/or has not provided Visa   encryptio')";
             $this->db->query($sql);
		} else if($event_type == 'module.disabled') {
			// TODO Handle actions when this module is disabled.
		} else if($event_type == 'module.enabled') {
			// TODO Handle actions when this module is enabled.
		} else if($event_type == 'module.preuninstall') {
			// TODO Handle actions when this module is about to be deleted.
		} else if($event_type == 'module.preupdate') {
			// TODO Handle actions before this module is updated.
		} else if($event_type == 'module.postupdate') {
			// TODO Handle actions after this module is updated.
		}
	}

	/** 
	 * Handle saving related module information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	// function save_related_module($module, $crmid, $with_module, $with_crmid) { }
	
	/**
	 * Handle deleting related module information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	//function delete_related_module($module, $crmid, $with_module, $with_crmid) { }

	/**
	 * Handle getting related list information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	//function get_related_list($id, $cur_tab_id, $rel_tab_id, $actions=false) { }

	/**
	 * Handle getting dependents list information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	//function get_dependents_list($id, $cur_tab_id, $rel_tab_id, $actions=false) { }
    /**
      Function to whether already exist Merchant account
      return array 
     */
    function checkMaccount()
    {
        $sql="select * from vtiger_pa_account";
        $result = $this->db->query($sql);
        $count = $this->db->getRowCount($result);
        if($count > 0 ) {
            $row=$this->db->fetch_row($result);
            $row['merchant_password']=$this->de_cryption($row['merchant_password']);
            return $row;
        }
        else
            return FALSE;
    }
     /**
      Funtion to insert CA merchant account
      @param array $data 
      return int
     */
    function addMaccount($data)
    {
        $crypt_pass=$this->encryption($_POST['pa_password']);
        $sql = "insert into vtiger_pa_account(account_id,merchant_username,
            merchant_password,currency) values ('','$data[pa_username]','$crypt_pass','$data[pa_currency]')";
        $result = $this->db->query($sql);
        if($result) {
            return TRUE;
        }
        else {
            return FALSE;
        }
    }
    function de_cryption($data)
    {
        require_once('include/utils/encryption.php');
        $de_crypt = new Encryption();
        if(isset($data))
        {
            $decrypted_password = $de_crypt->decrypt($data);
        }
        return $decrypted_password;
    }
    /**
      Function to encypt string
      @param string
      return string
     */
    function encryption($newpassword)
    {
        require_once('include/utils/encryption.php');
        $en_crypt = new Encryption();
        if( isset($newpassword))
        {
            $encrypted_password = $en_crypt->encrypt($newpassword);
        }

        return $encrypted_password;
    }
     /**
      Function to update Merchant account details
      @param array update data
      return int TRUE/FALSE 
     */
    function editMaccount($data)
    {
        $crypt_pass=$this->encryption($_POST['pa_password']);
        $sql = "update vtiger_pa_account set merchant_username='$data[pa_username]',
            merchant_password='$crypt_pass',currency='$data[pa_currency]' where account_id='$data[accountid]'";
        $result = $this->db->query($sql);
        if($result) {
            return TRUE;
        }
        else {
            return FALSE;
        }
    }
     /**
      Function to submit transaction form to CA and save response 
      @param array form data
      @param string transaction_type
      @param string payment_type
      return response array if successfull otherwise FALSE
     */
    function submitForm($data,$type,$payment="CC")
    {
        global $current_user;
        $account = $this->checkMaccount();
        //$data['ccexp']=$data['ccexpmm'].$data['ccexpyy'];
        $data['username']=$account['merchant_username'];
        $data['password']=$account['merchant_password'];
        $data['type']=$type;
        $string="";
        foreach($data as $key=>$value) {
            $encoded = urlencode($value);
            $string=$string."$key=$encoded".'&';
        }
        $result = $this->doPost($string);
        if($result) {
            $this->addTransaction($result,$payment,$type,$data['amount'],$data['parent_id']);
            return $result;
        }
        else
            return FALSE;
    }
    function addTransaction($result,$payment,$type,$amount='',$parent_id='',$order='',$recurring='')
    {
            global $current_user;
            parse_str($result);
            $date=date("Y-m-d");
            if ($type == "capture" || $type == "void" || $type == "refund" ) { 
                $parent_id = $this->getParentid($transactionid);
                $this->setData($transactionid);
            }
            if($transactionid) {
            $this->column_fields[transaction_id]=$transactionid;
            $this->column_fields[payment_type]=$payment;
            $this->column_fields[transaction_type]=$type;
            $this->column_fields[response]=$response;
            $this->column_fields[responsetext]=$responsetext;
            $this->column_fields[authcode]=$authcode;
            $this->column_fields[avsresponse]=$avsresponse;
            $this->column_fields[cvvresponse]=$cvvresponse;
            $this->column_fields[orderid]=$orderid;
            $this->column_fields[response_code]=$response_code;
            $this->column_fields[assigned_user_id]=$current_user->id;
            $this->column_fields[parent_id]=$parent_id;
            $this->column_fields[transaction_date]=$date;
            $this->column_fields[transaction_amount]=$amount;
            $this->column_fields[description]=$order;
            $this->column_fields[customer_vault_id]=$customer_vault_id;
            $this->column_fields[recurring_plans]=$recurring;
            $this->save("Planetauthorize");
            }        
    }
    //Function for generating Credit Sale request
    function doSale($data)
    {
        $account = $this->checkMaccount();    
        $query  = "";
        $query .= "username=" . urlencode($account['merchant_username']) . "&";
        $query .= "password=" . urlencode($account['merchant_password']) . "&";
        $query .= "ccnumber=" . urlencode($data['ccnumber']) . "&";
        $query .= "ccexp=" . urlencode($data['ccexp']) . "&";
        $query .= "amount=" . urlencode($data['amount']) . "&";
        $query .= "cvv=" . urlencode($data['cvv']) . "&";
        $query .= "currency=" . urlencode($data['currency']) . "&";
        // Order Information
        $query .= "orderid=" . urlencode($data['orderid']) . "&";
        $query .= "orderdescription=" . urlencode($data['orderdescription']) . "&";
        $query .= "tax=" . urlencode($data['tax']) . "&";
        $query .= "shipping=" . urlencode($data['shipping']) . "&";
        $query .= "ponumber=" . urlencode($data['ponumber']) . "&";
        // Billing Information
        $query .= "firstname=" . urlencode($data['firstname']) . "&";
        $query .= "lastname=" . urlencode($data['lastname']) . "&";
        $query .= "company=" . urlencode($data['company']) . "&";
        $query .= "address1=" . urlencode($data['bill_street']) . "&";
        $query .= "address2=" . urlencode($data['address2']) . "&";
        $query .= "city=" . urlencode($data['bill_city']) . "&";
        $query .= "state=" . urlencode($data['bill_state']) . "&";
        $query .= "zip=" . urlencode($data['bill_code']) . "&";
        $query .= "country=" . urlencode($data['country']) . "&";
        $query .= "phone=" . urlencode($data['phone']) . "&";
        $query .= "fax=" . urlencode($data['fax']) . "&";
        $query .= "email=" . urlencode($data['email']) . "&";
        $query .= "website=" . urlencode($data['website']) . "&";
        // Shipping Information
        $query .= "shipping_firstname=" . urlencode($data['shipping_firstname']) . "&";
        $query .= "shipping_lastname=" . urlencode($data['shipping_lastname']) . "&";
        $query .= "shipping_company=" . urlencode($data['shipping_company']) . "&";
        $query .= "shipping_address1=" . urlencode($data['shipping_address1']) . "&";
        $query .= "shipping_address2=" . urlencode($data['shipping_address2']) . "&";
        $query .= "shipping_city=" . urlencode($data['shipping_city']) . "&";
        $query .= "shipping_state=" . urlencode($data['shipping_state']) . "&";
        $query .= "shipping_zip=" . urlencode($data['shipping_zip']) . "&";
        $query .= "shipping_country=" . urlencode($data['shipping_country']) . "&";
        $query .= "shipping_email=" . urlencode($data['shipping_email']) . "&";
        $query .= "payment=" . urlencode($data['payment']) . "&";
        //Check if customer vault is set
        if(isset($data['add_vault']))
        $query .= "customer_vault=add_customer&cutomer_vault_id=".urlencode($data['customer_vault_id'])."&";
        //Recurring billing
        if(isset($data['billing']))
        {
            if($data['product_sku']) {
            $length = count($data['product_sku'])<5?count($data['product_sku']):5;
            $index = $i+1;
            $recurring = implode(',',$data['product_sku']);
            for($i=0;$i<$length;$i++)
                $query .= "product_sku_$index=" . urlencode($data['product_sku'][$i]) . "&";
            }       
        }
        $query .= "type=sale";
        $result = $this->doPost($query);
        if($result) {
            $this->setAddress($data);
            $this->addTransaction($result,'CC','sale',$data['amount'],$data['parent_id'],$data['orderdescription'],$recurring);
            return $result;
        }
        return false;
    }
    //Function for generating Credit Auth request
     function doAuth($data)
    {
        $account = $this->checkMaccount();
        $query  = "";
        $query .= "username=" . urlencode($account['merchant_username']) . "&";
        $query .= "password=" . urlencode($account['merchant_password']) . "&";
        $query .= "ccnumber=" . urlencode($data['ccnumber']) . "&";
        $query .= "ccexp=" . urlencode($data['ccexp']) . "&";
        $query .= "amount=" . urlencode($data['amount']) . "&";
        $query .= "cvv=" . urlencode($data['cvv']) . "&";
        $query .= "currency=" . urlencode($data['currency']) . "&";
        // Order Information
        $query .= "orderid=" . urlencode($data['orderid']) . "&";
        $query .= "orderdescription=" . urlencode($data['orderdescription']) . "&";
        $query .= "tax=" . urlencode($data['tax']) . "&";
        $query .= "shipping=" . urlencode($data['shipping']) . "&";
        $query .= "ponumber=" . urlencode($data['ponumber']) . "&";
        // Billing Information
        $query .= "firstname=" . urlencode($data['firstname']) . "&";
        $query .= "lastname=" . urlencode($data['lastname']) . "&";
        $query .= "company=" . urlencode($data['company']) . "&";
        $query .= "address1=" . urlencode($data['bill_street']) . "&";
        $query .= "address2=" . urlencode($data['address2']) . "&";
        $query .= "city=" . urlencode($data['bill_city']) . "&";
        $query .= "state=" . urlencode($data['bill_state']) . "&";
        $query .= "zip=" . urlencode($data['bill_code']) . "&";
        $query .= "country=" . urlencode($data['country']) . "&";
        $query .= "phone=" . urlencode($data['phone']) . "&";
        $query .= "fax=" . urlencode($data['fax']) . "&";
        $query .= "email=" . urlencode($data['email']) . "&";
        $query .= "website=" . urlencode($data['website']) . "&";
        // Shipping Information
        $query .= "shipping_firstname=" . urlencode($data['shipping_firstname']) . "&";
        $query .= "shipping_lastname=" . urlencode($data['shipping_lastname']) . "&";
        $query .= "shipping_company=" . urlencode($data['shipping_company']) . "&";
        $query .= "shipping_address1=" . urlencode($data['shipping_address1']) . "&";
        $query .= "shipping_address2=" . urlencode($data['shipping_address2']) . "&";
        $query .= "shipping_city=" . urlencode($data['shipping_city']) . "&";
        $query .= "shipping_state=" . urlencode($data['shipping_state']) . "&";
        $query .= "shipping_zip=" . urlencode($data['shipping_zip']) . "&";
        $query .= "shipping_country=" . urlencode($data['shipping_country']) . "&";
        $query .= "shipping_email=" . urlencode($data['shipping_email']) . "&";
        $query .= "payment=" . urlencode($data['payment']) . "&";
        if(isset($data['add_vault']))
        $query .= "customer_vault=add_customer&cutomer_vault_id=".urlencode($data['customer_vault_id'])."&";
        $query .= "type=auth";
        $result = $this->doPost($query);
        if($result) {
            $this->setAddress($data);
            $this->addTransaction($result,'CC','auth',$data['amount'],$data['parent_id'],$data['orderdescription']);
        return $result;
        }
        return false;
    }
    //Function for generating Eletronic Check Sale request query
    function doECSale($data)
    {
        $account = $this->checkMaccount();
        $query  = "";
        $query .= "username=" . urlencode($account['merchant_username']) . "&";
        $query .= "password=" . urlencode($account['merchant_password']) . "&";
        $query .= "checkname=" . urlencode($data['checkname']) . "&";
        $query .= "checkaba=" . urlencode($data['checkaba']) . "&";
        $query .= "checkaccount=" . urlencode($data['checkaccount']) . "&";
        $query .= "account_type=" . urlencode($data['account_type']) . "&";
        $query .= "sec_code=" . urlencode($data['sec_code']) . "&";
        $query .= "amount=" . urlencode($data['amount']) . "&";
        $query .= "currency=" . urlencode($data['currency']) . "&";
         // Order Information
        $query .= "orderid=" . urlencode($data['orderid']) . "&";
        $query .= "orderdescription=" . urlencode($data['orderdescription']) . "&";
        $query .= "tax=" . urlencode($data['tax']) . "&";
        $query .= "shipping=" . urlencode($data['shipping']) . "&";
        $query .= "ponumber=" . urlencode($data['ponumber']) . "&";
         // Billing Information
        $query .= "firstname=" . urlencode($data['firstname']) . "&";
        $query .= "lastname=" . urlencode($data['lastname']) . "&";
        $query .= "company=" . urlencode($data['company']) . "&";
        $query .= "address1=" . urlencode($data['bill_street']) . "&";
        $query .= "address2=" . urlencode($data['address2']) . "&";
        $query .= "city=" . urlencode($data['bill_city']) . "&";
        $query .= "state=" . urlencode($data['bill_state']) . "&";
        $query .= "zip=" . urlencode($data['bill_code']) . "&";
        $query .= "country=" . urlencode($data['country']) . "&";
        $query .= "phone=" . urlencode($data['phone']) . "&";
        $query .= "fax=" . urlencode($data['fax']) . "&";
        $query .= "email=" . urlencode($data['email']) . "&";
        $query .= "website=" . urlencode($data['website']) . "&";
        // Shipping Information
        $query .= "shipping_firstname=" . urlencode($data['shipping_firstname']) . "&";
        $query .= "shipping_lastname=" . urlencode($data['shipping_lastname']) . "&";
        $query .= "shipping_company=" . urlencode($data['shipping_company']) . "&";
        $query .= "shipping_address1=" . urlencode($data['shipping_address1']) . "&";
        $query .= "shipping_address2=" . urlencode($data['shipping_address2']) . "&";
        $query .= "shipping_city=" . urlencode($data['shipping_city']) . "&";
        $query .= "shipping_state=" . urlencode($data['shipping_state']) . "&";
        $query .= "shipping_zip=" . urlencode($data['shipping_zip']) . "&";
        $query .= "shipping_country=" . urlencode($data['shipping_country']) . "&";
        $query .= "shipping_email=" . urlencode($data['shipping_email']) . "&";
        $query .= "payment=" . urlencode($data['payment']) . "&";
        //Check if customer vault is set
        if(isset($data['add_vault']))
        $query .= "customer_vault=add_customer&cutomer_vault_id=".urlencode($data['customer_vault_id'])."&";
         //Recurring billing
        if(isset($data['billing']))
        {
            if($data['product_sku']) {
            $length = count($data['product_sku'])<5?count($data['product_sku']):5;
            $index = $i+1;
            $recurring = implode(',',$data['product_sku']);
            for($i=0;$i<$length;$i++)
                $query .= "product_sku_$index=" . urlencode($data['product_sku'][$i]) . "&";
            }
        }
        $query .= "type=sale";
        $result = $this->doPost($query);
        if($result) {
            $this->setAddress($data);;
            $this->addTransaction($result,'CHECK','sale',$data['amount'],$data['parent_id'],$data['orderdescription'],$recurring);
            return $result;
        }
        return false;
    }
    function getAVS($avscode) {
        $sql = "select value from vtiger_pa_avsresponsecode where code='$avscode'";
        $result = $this->db->query($sql);
        if($result) {
            $arr = $this->db->fetch_row($result);
            return $arr['value'];
        }
        else
            return "";
    }
    /**
      Function to get CVV string from cvvcode
      @param int cvvcode
      return string
     */
    function getCVV($cvvcode) {
        $sql = "select value from vtiger_pa_cvvresponsecode where code='$cvvcode'";
        $result = $this->db->query($sql);
        if($result) {
            $arr = $this->db->fetch_row($result);
            return $arr['value'];
        }
        else
            return "";
    }
     /**
      Function to get parent_id for Capture,Void,Refund transactions
      @param int transactionid
      return parent_id
     */
    function getParentid($transactionid){
        $sql = "select parent_id from vtiger_pa_transactions where transaction_id='$transactionid'";
        $result = $this->db->query($sql);
        if($result) {
            $arr = $this->db->fetch_row($result);
            return $arr['parent_id'];
        }
        else
            return "";
    }
    function getResponse($id) {
        $sql = "select value from vtiger_pa_response where id='$id'";
        $result = $this->db->query($sql);
        if($result) {
            $arr = $this->db->fetch_row($result);
            return $arr['value'];
        }
        else
            return "";

    }
    //Customized fucntion to filter list query fields
    function getModifiedListQuery($listquery,&$customView,$viewid)
    {
        $columnslist = $customView->getColumnsListByCvid($viewid);
        foreach($columnslist as $columnname=>$value)
        {
            $tablefield = "";
            if($value != "")
            {
                $list = explode(":",$value);
                $sqllist_column = $list[0].".".$list[1];
                if($list[1] == "smownerid")
                {
                    $sqllist_column = "case when (vtiger_users.user_name not like '') then vtiger_users.user_name else vtiger_groups.groupname end as user_name";
                }
                if($list[1] == "response") {
                    $sqllist_column = "vtiger_pa_response.value as response ";
                }
                $sqllist[] = $sqllist_column;
            }
            $tablefield[$list[0]] = $list[1];
            $sql = implode(",",$sqllist);
            $tablefield[$list[0]] = $list[1];

            //Changed as the replace of module name may replace the string if the fieldname has module name in it -- Jeri
            $fieldinfo = explode('_',$list[3],2);
            $fieldlabel = $fieldinfo[1];
            $fieldlabel = str_replace("_"," ",$fieldlabel);

            if($customView->isFieldPresent_ByColumnTable($list[1], $list[0])){

                $customView->list_fields[$fieldlabel] = $tablefield;
                $customView->list_fields_name[$fieldlabel] = $list[2];
            }


        }
        $listviewquery = substr($listquery, strpos($listquery,'FROM'),strlen($listquery));
        $list_query = "select ".$sql." ,vtiger_crmentity.crmid ".$listviewquery;
        return  $list_query;
    }
    function getTransaction($id) {
        $sql = "select $this->table_name.*,vtiger_pa_response.value,  
            vtiger_pa_avsresponsecode.Value as avs, vtiger_pa_cvvresponsecode.value as cvv  
            from $this->table_name 
            left join vtiger_pa_response on $this->table_name.response=vtiger_pa_response.id 
            left join vtiger_pa_avsresponsecode on $this->table_name.avsresponse = vtiger_pa_avsresponsecode.Code
            left join vtiger_pa_cvvresponsecode on $this->table_name.cvvresponse = vtiger_pa_cvvresponsecode.code
            where $this->table_name.planetauthorizeid=?";
        $result = $this->db->pquery($sql,array($id));
        if($result)
            return $this->db->fetch_row($result);
        else
            return FALSE;
    }
    //Function for submit POST request to payment gateway
    function doPost($query)
    {
        $ch  = curl_init();
        curl_setopt($ch, CURLOPT_URL,$this->cgiurl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,true);
        curl_setopt($ch, CURLOPT_TIMEOUT,30);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS,$query);
        $result= curl_exec($ch);
        curl_close($ch);
        return $result;       
    }
    function addPlan($data)
    {
        $sql = "update vtiger_pa_recurringplan set plan=?,sku=?,cycle=?,amount=? where id=?";
        $sql = "insert into vtiger_pa_recurringplan(id,plan,sku,cycle,amount) values (?,?,?,?,?)";
        $result = $this->db->pquery($sql,array('',$data['plan'],$data['sku'],$data['cycle'],$data['amount']));
        if($result) 
            return TRUE;
        else 
            return FALSE;
    }
    function editPlan($id,$data)
    {
        $sql = "update vtiger_pa_recurringplan set plan=?,sku=?,cycle=?,amount=? where id=?";
        $result = $this->db->pquery($sql,array($data['plan'],$data['sku'],$data['cycle'],$data['amount'],$id));
        if($result)
        return true;
        else
        return false;
    }
    function listPlan()
    {
        $result_arr = array();
        $sql = "select * from vtiger_pa_recurringplan";
        $result = $this->db->query($sql);
        if($result) {
            while($row = $this->db->fetch_array($result))
            array_push($result_arr,$row);
            return $result_arr;    
        }
        else
        return false;
        
    }
    function getPlan($id)
    {
        $sql = "select * from vtiger_pa_recurringplan where id=?";
        $result = $this->db->pquery($sql,array($id));
        if($result) 
        return $this->db->fetch_array($result);
        else
        return false;

    }
    function deletePlan($id)
    {
        $sql = "delete from vtiger_pa_recurringplan where id=?";
        $result = $this->db->pquery($sql,array($id));
        if($result)
        return true;
        else
        return false;
    }
    //New sale using customer vault
    function doCVSale($data)
    {
        global $current_user;
        $payment=$data['payment']=='creditcard'?'CC':'CHECK';  
        $account = $this->checkMaccount();
        $query  = "";
        $query .= "username=" . urlencode($account['merchant_username']) . "&";
        $query .= "password=" . urlencode($account['merchant_password']) . "&";
        $query .= "amount=" . urlencode($data['amount']) . "&";
        $query .= "customer_vault_id=" . urlencode($data['customer_vault_id']);
        $result = $this->doPost($query);
        if($result) {
        $this->addTransaction($result,$payment,'sale',$data['amount']);
        return $result;
        }
        return false;
    }
    //Update customer vault information
    function doCVUpdate($data)
    {
        $account = $this->checkMaccount();
        $query  = "";
        $query .= "username=" . urlencode($account['merchant_username']) . "&";
        $query .= "password=" . urlencode($account['merchant_password']) . "&";
        $query .= "customer_vault=update_customer&customer_vault_id=". urlencode($data['customer_vault_id']);
        $query .= "ccnumber=" . urlencode($data['ccnumber']) . "&";
        $query .= "ccexp=" . urlencode($data['ccexp']);
        $result = $this->doPost($query); 
        return $result;
    }
    function DeleteCPlan($record)
    {
        $account = $this->checkMaccount();
        $query  = "";
        $query .= "username=" . urlencode($account['merchant_username']) . "&";
        $query .= "password=" . urlencode($account['merchant_password']) . "&";
        $query .= "delete_recurring=" . urlencode($record);
        $result = $this->doPost($query);
        if($result) {
            $sql = "update $this->table_name set recurring_plans='' where transaction_id=?";
            $this->db->pquery($sql,array($record));
        }
        return $result;
    }
    function setAddress($data)
    {
        $this->column_fields['bill_firstname'] = $data['firstname'];
        $this->column_fields['bill_lastname'] = $data['lastname'];
        $this->column_fields['bill_address'] = $data['bill_street']." ". $data['address2'];
        $this->column_fields['bill_city'] = $data['bill_city'];
        $this->column_fields['bill_state'] = $data['bill_state'];
        $this->column_fields['bill_zip'] = $data['bill_code'];
        $this->column_fields['bill_country'] = $data['country'];
        //shipping
        $this->column_fields['ship_firstname'] = $data['shipping_firstname'];
        $this->column_fields['ship_lastname'] = $data['shipping_lastname'];
        $this->column_fields['ship_address'] = $data['shipping_address1']." ". $data['shipping_address2'];
        $this->column_fields['ship_city'] = $data['shipping_city'];
        $this->column_fields['ship_state'] = $data['shipping_state'];
        $this->column_fields['ship_zip'] = $data['shipping_zip'];
        $this->column_fields['ship_country'] = $data['shipping_country'];
    }
    function setData($transactionid)
    {
        $sql = "select * from $this->table_name where transaction_id=?";
        $result = $this->db->pquery($sql,array($transactionid));
        if($result) {
            $data = $this->db->fetch_array($result);
            $this->column_fields['bill_firstname'] = $data['bill_firstname'];
            $this->column_fields['bill_lastname'] = $data['bill_lastname'];
            $this->column_fields['bill_address'] = $data['bill_address'];
            $this->column_fields['bill_city'] = $data['bill_city'];
            $this->column_fields['bill_state'] = $data['bill_state'];
            $this->column_fields['bill_zip'] = $data['bill_zip'];
            $this->column_fields['bill_country'] = $data['bill_country'];
            //shipping
            $this->column_fields['ship_firstname'] = $data['ship_firstname'];
            $this->column_fields['ship_lastname'] = $data['ship_lastname'];
            $this->column_fields['ship_address'] = $data['ship_address'];
            $this->column_fields['ship_city'] = $data['ship_city'];
            $this->column_fields['ship_state'] = $data['ship_state'];
            $this->column_fields['ship_zip'] = $data['ship_zip'];
            $this->column_fields['ship_country'] = $data['ship_country'];

        }

    }
    function getTransactionByType($transaction_id,$transaction_type)
    {
        $sql = "select * from $this->table_name where transaction_id=? and transaction_type=?";
        $result = $this->db->pquery($sql,array($transaction_id,$transaction_type));
        if($result) 
             return $this->db->fetch_array($result);
        else
        return false;
    }
}
?>
